#include <cassert>

#include "App.h"
#include "Engine.h"
#include "HallOfFame.h"
#include "ScoreSubmit.h"


void App::ProcessEvents() {
    if (is_done) {
        return;
    }

    // przyjrzyj zdarzenia
    SDL_Event event;
    while (SDL_PollEvent(&event)) {
        if (event.type == SDL_VIDEORESIZE) {
            Resize(event.resize.w, event.resize.h);
        } else if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_0) {

            m_game.reset(new Game());
            m_game->Init();

        } else {
            m_game->ProcessEvents(event);
        }
    }
}

void App::Run() {
    // inicjalizacja okna
    SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO);
    Resize(m_window_width, m_window_height);
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1); // podwójne buforowanie

    Engine::Get().Sound()->LoadSounds();

    // inicjalizacja OpenGL
    glClearColor(0, 0, 0, 0);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_ALPHA_TEST); // niewyświetlanie przezroczystych fragmentów sprite'a
    glAlphaFunc(GL_GEQUAL, 0.1);


    m_game.reset(new Game());
    m_game->Init();
    m_game->Start();


//     // pętla główna
//     is_done = false;
//     size_t last_ticks = SDL_GetTicks();
//     while (!is_done) {
//         ProcessEvents();

//         // time update
//         size_t ticks = SDL_GetTicks();
//         double delta_time = (ticks - last_ticks) / 1000.0;
//         last_ticks = ticks;

//         // update & render
//         if (delta_time > 0) {
//             Update(delta_time);
//         }
//         Draw();
//     }

//     // pętla główna
//     HallOfFame hof;
//     is_done = false;
//     size_t last_ticks = SDL_GetTicks();
//     while (!hof.IsDone()) {
//         hof.ProcessEvents();

//         // time update
//         size_t ticks = SDL_GetTicks();
//         double delta_time = (ticks - last_ticks) / 1000.0;
//         last_ticks = ticks;

//         // update & render
//         if (delta_time > 0) {
//             hof.Update(delta_time);
//         }
//         hof.Draw();
//     }


    // // pętla główna
    // ScoreSubmit ss(102);
    // is_done = false;
    // size_t last_ticks = SDL_GetTicks();
    // while (!ss.IsDone()) {
    //     ss.ProcessEvents();

    //     // time update
    //     size_t ticks = SDL_GetTicks();
    //     double delta_time = (ticks - last_ticks) / 1000.0;
    //     last_ticks = ticks;

    //     // update & render
    //     if (delta_time > 0) {
    //         ss.Update(delta_time);
    //     }
    //     ss.Draw();
    // }


    // pętla główna
    is_done = false;
    size_t last_ticks = SDL_GetTicks();
    while (m_game->IsDone() == false) {
        ProcessEvents();

        // time update
        size_t ticks = SDL_GetTicks();
        double delta_time = (ticks - last_ticks) / 1000.0;
        last_ticks = ticks;

        // update & render
        if (delta_time > 0) {
            Update(delta_time);
        }
        Draw();
    }

    SDL_Quit();
}

void App::Update(double dt) {
    if (m_game->Update(dt) == false) {
        is_done = true;
    }
}

void App::Draw() {
    m_game->Draw();
}

void App::Resize(size_t width, size_t height) {
    m_screen = SDL_SetVideoMode(width, height, 32, SDL_OPENGL | SDL_RESIZABLE | SDL_HWSURFACE);
    assert(m_screen && "problem z ustawieniem wideo");
    m_window_width = width;
    m_window_height = height;

    Engine::Get().Renderer()->SetProjection(width, height);
}
