#include <iostream>
#include "Engine.h"
#include "MushEntity.h"
#include "PlayerBulletEntity.h"
#include "EntityFactory.h"


EntityFactory::EntityFactory() {
}

SpritePtr GetSpriteByName(const std::string& name) {
    return SpritePtr(new Sprite(Engine::Get().SpriteConfig()->Get(name)));
}

EntityPtr EntityFactory::CreateEntity(ET::EntityType type, double x, double y) {
    const double eps = 0.0001;
    x += eps;  y += eps;

    EntityPtr ptr;
    if(type==ET::Mush) {
//        std::cout << "[EntityFactory] Create Mush: " << x << ", " << y << std::endl;
        ptr.reset(new MushEntity(x, y));
        ptr->SetSprites(GetSpriteByName("mush_left"), GetSpriteByName("mush_right"), GetSpriteByName("mush_stop") );
    } else if (type == ET::PlayerBullet) {
//        std::cout << "[EntityFactory] Create PlayerBullet: " << x << ", " << y << std::endl;
        ptr.reset(new PlayerBulletEntity(x, y));
        SpritePtr bullet = GetSpriteByName("player_bullet"); //(new Sprite(engine.SpriteConfig()->Get("player_bullet")));
        ptr->SetSprites(bullet, bullet, bullet);
    }

    if (!ptr) {
        std::cerr << "fabryka nie umie stworzyć żądanej jednostki: " << type << ", " << x << ", " << y << std::endl;
    }
    return ptr;
}


EntityPtr EntityFactory::CreateEntity(const std::string& name, double x, double y) {
    if (name == "mush") {
        return CreateEntity(ET::Mush, x, y);
    } else if (name == "player_bullet") {
        return CreateEntity(ET::PlayerBullet, x, y);
    }
    std::cerr << "fabryka nie umie stworzyć żądanej jednostki: " << name << std::endl;
    return EntityPtr();
}
