#ifndef __GAME_H_INCLUDED__
#define __GAME_H_INCLUDED__
#include <boost/shared_ptr.hpp>
#include <SDL/SDL.h>
#include <vector>

#include "Player.h"
#include "Object.h"
#include "Level.h"
#include "SpriteGrid.h"
#include "Entity.h"

#include <iostream>

class Game {
public:
    explicit Game() 
        : m_is_done(false),
          m_stored_player_pos_x(1.0) {
    }

    void Start();
    void Init();
    void Draw();
    bool Update(double dt);
    void ProcessEvents(const SDL_Event& event);
    bool IsDone() const { return m_is_done; }

    void ExecuteCreators();
    bool CanAddEntity(EntityPtr entity) const {
        size_t curr_tile_x, curr_tile_y;
        entity->GetCurrentTile(&curr_tile_x, &curr_tile_y);
        if (m_level->Field(curr_tile_x, curr_tile_y) != FT::None) {
            return false;
        }
        return true;
    }
    void AddEntity(EntityPtr entity) {
        m_entities.push_back(entity);
    }

    LevelPtr GetLevel() const { return m_level; }

private:
    void SeepAndAddEntities(double dt);
    void CheckPlayerEntitiesCollisions(double dt);
    void CheckEntityEntityCollisions(double dt);
    void CheckCollisionOfOnePair(EntityPtr fst_entity, ET::EntityType fst_type,
                                 EntityPtr snd_entity, ET::EntityType snd_type, double dt);

    bool m_is_done;        // czy należy zakończyć grę

    PlayerPtr m_player;
    std::vector<EntityPtr> m_entities;                // jednostki w grze
    std::list<LevelEntityData> m_entities_to_create;  // opisy jednostek do stworzenia

    LevelPtr m_level;
    SpriteGrid m_level_view;
    double m_stored_player_pos_x;
};

typedef boost::shared_ptr<Game> GamePtr;

#endif
